----------------------------------------------------------------------
-- $B=PNOMQ(B
--
module Format where {
import Data.Char (isAlpha);
import TIMain (Expr(Var, Lit, Const, Ap, Let, Lam, If, Case));
import Lit;
import Pat;
import Type; 
import Scheme;
import Pred;
import Parser (Decl(GenDecl,Decl,EmptyDecl),RawTopDecl(RawVariantDecl, RawRecordDecl, RawInstDecl, RawTopDecl));
import PPrint (pprint, render, char, text, hsep, vcat, empty, parens, braces, (<+>), (<>), ($$), punctuate);

-- import PPrint;
formatImpl (id, alts) =
  vcat (map (\ (pats, e) -> text id <+>
	                    formatPats pats <+> 
	                    text "=" <+>
	                    formatExpr e <> char ';')
	alts);

formatImpls es = vcat (map formatImpl es);

formatInnerInstDecl (ps :=> p, impls) =
  text "instance" <+>
  formatPreds ps <+>
  formatPred p <+>
  text "where" <+>
  braces (vcat (map formatImpl impls)) <> char ';'
;

formatInnerInstDecls iis = vcat (map formatInnerInstDecl iis);

formatScheme (Forall _ (ps :=> t)) = formatQualType (ps :=> t);

formatQualType (ps :=> t) = formatPreds ps <+> formatType t;

formatExpl (id, scm, alts) =
    (text id <+> text "::" <+> formatScheme scm <> char ';') 
    $$
    formatImpl (id, alts);

formatExpls es = vcat (map formatExpl es);

printExpl (id, scm, alts) = do {
                                putStr id;
                                putStr " :: ";
                                putStr $ render $ pprint scm;
                                putStrLn "\n";
                                print alts
                            };

parensIf b d = if b then parens d else d;

isTupleIdAux [c]     = c==')';
isTupleIdAux (c:str) = c==',' && isTupleIdAux str;
isTupleIdAux []      = False;
isTupleId (c:str)    = c=='(' && isTupleIdAux str; 

-- formatRawType t = text (show t);
formatRawTypeAux _ (Var id)  = text id;
formatRawTypeAux _ (Const c) = text c;
formatRawTypeAux d app = 
 let {
   sepArgs (Ap f a) acc = sepArgs f (a:acc);
   sepArgs f        acc = (f, acc);
 } in case sepArgs app [] of {
   (Const "(->)", [from, to])  ->
     parensIf (d>=1) (formatRawTypeAux 1 from <+> 
                      text "->" <+> 
                      formatRawTypeAux 0 to);
   (Const id, ts)  | isTupleId id ->
       parens (hsep (punctuate (text ",") (map (formatRawTypeAux 0) ts)));
   (f, ts) ->
     parensIf (d>=2) (formatRawTypeAux 2 f <+>
		      hsep (map (formatRawTypeAux 2) ts))
 }
; 

formatRawType t = formatRawTypeAux 0 t;

formatDecl (GenDecl id (ps, t)) = 
  text id <+> text "::" <+> formatRawPreds ps <+> formatRawType t <> char ';';
formatDecl (Decl id (pats, e)) = text id <+> formatPats pats <+> text "=" <+> formatExpr e;
formatDecl EmptyDecl     = empty;      

formatRawPred (c, vs) = text c <+> hsep (map text vs);

formatRawPreds []  = empty;
formatRawPreds [p] = formatRawPred p <+> text "=>"; 
formatRawPreds ps  = parens (hsep (punctuate (text ",") (map formatRawPred ps))) <+> text "=>";

formatDependency (c, [v]) = empty;
formatDependency (c, v:vs) =
   char '|' <+> hsep (punctuate (text ",") (map (\ w -> text v <+> text "->" <+> text w) vs));

formatClassDeclAux ps p ds flag =
  text "class" <+>
  text (if flag then "{- variant -}" else  "{- record -} ") <+>
  formatRawPreds ps <+>
  formatRawPred  p <+>
  formatDependency p <+> 
  text "where" <+>
  braces (vcat (map formatDecl ds)) <>  
  char ';'
;

formatClassDecl (RawVariantDecl ps p ds)  = formatClassDeclAux ps p ds True;
formatClassDecl (RawRecordDecl ps p ds)   = formatClassDeclAux ps p ds False;
formatClassDecl (RawInstDecl (rps, (vrp, id, rrp)) decls)    =
    text "instance" <+> formatRawPreds rps  <+>
    formatRawPred vrp <+> text "::" <+> text id <+> text "::" <+> formatRawPred rrp <+>
    braces (vcat (map formatDecl decls));
formatClassDecl (RawTopDecl decl)         = formatDecl decl;     

formatClassDecls ds = vcat (map formatClassDecl ds);

formatTypeAux _ (TVar tv) = formatTyvar tv;
formatTypeAux _ (TCon tc) = formatTycon tc;
formatTypeAux _ (TGen i)  = text ("v" ++ show i);
formatTypeAux d app       = 
 let {
   sepArgs (TAp f a) acc = sepArgs f (a:acc);
   sepArgs f        acc = (f, acc);

 } in case sepArgs app [] of {
   (TCon (Tycon "(->)" _), [from, to])  ->
     parensIf (d>=1) (formatTypeAux 1 from <+> 
                      text "->" <+> 
                      formatTypeAux 0 to);
   (TCon (Tycon id _), ts)  | isTupleId id ->
       parens (hsep (punctuate (text ",") (map (formatTypeAux 0) ts)));
   (f, ts) ->
       parensIf (d>=2) (formatTypeAux 2 f <+> 
  	                    hsep (map (formatTypeAux 2) ts))
 }
; 

formatType t = formatTypeAux 0 t; 

formatPred (IsIn id ts) = hsep (text id : map (formatTypeAux 2) ts);

formatPreds []  = empty;
formatPreds [p] = formatPred p <+> text "=>";
formatPreds ps  = parens (hsep (map formatPred ps)) <+> text "=>"; 

formatForall [] = empty;
formatForall vs = 
  text "forall" <+>
  hsep (map formatTyvar vs) <+>        
  text "."
;

formatInnerConstDecl (id, (vs, ts)) = 
  formatForall vs <+>
  text id <+>
  hsep (map (formatTypeAux 2) ts)
;

formatInnerConstDecls [d] = formatInnerConstDecl d;
formatInnerConstDecls ds  = vcat (punctuate (text " | ") (map formatInnerConstDecl ds));

formatTyvar (Tyvar id _) = text id;
formatTycon (Tycon id _) = text id;

formatInnerDataDecl ((id, vs, mdecls, gdecls)) =
  case mdecls of {
    Just (ps :=> cdecls) -> 
      text "data" <+>
      formatPreds ps <+>
      text id <+>
      hsep (map formatTyvar vs) <+>
      char '=' <+>
      formatInnerConstDecls cdecls <+> 
      char ';' $$
      text "{-" $$
      text "-- using GADT-style declaration" $$      
      text "data" <+>
      text id <+>
      hsep (map formatTyvar vs) <+> 
      text "where" <+>
      braces (vcat (map (\ (id, qt) ->  text id <+> text "::" <+> formatQualType qt <> char ';') gdecls)) <+>
      char ';'$$
      text "-}"; 
    Nothing ->
     text "data" <+>
     text id <+>
     hsep (map formatTyvar vs) <+> 
     text "where" <+>
     braces (vcat (map (\ (id, qt) ->  text id <+> text "::" <+> formatQualType qt <> char ';') gdecls)) <+>
     char ';'
  }
;

formatInnerDataDecls ds = vcat (map formatInnerDataDecl ds);

formatBindGroup (es, iis) =
    formatExpls es $$
    vcat (map formatImpls iis);

formatBindGroups bgs = vcat (map formatBindGroup bgs);

formatLiteral (LitInt i) = text (show i);
formatLiteral (LitBool b) = text (show b);
formatLiteral (LitChar c) = text (show c);
formatLiteral (LitStr s) = text (show s);
formatLiteral (LitRat r) = parens (text (show r));

idName (c:str) = isAlpha c || c=='_';
textIdOrOp id = if idName id || isTupleId id then text id else parens (text id);

formatPatAux _ (PVar id)  = textIdOrOp id;
formatPatAux _ PWildcard  = char '_';
formatPatAux d (PAs id p)  = 
   parensIf (d>=11) (text id <> char '@' <> formatPatAux 10 p);
formatPatAux _ (PLit l)   = formatLiteral l;
formatPatAux d (PCon id pats) =
    if isTupleId id then
       parens (hsep (punctuate (text ",") (map (formatPatAux 0) pats)))
    else
       parensIf (d>=10) (textIdOrOp id <+> hsep (map (formatPatAux 10) pats)) ;
formatPatAux d (PLazy p)  =
   parensIf (d>=11) (char '~' <> formatPatAux 11 p);  

formatPat p = formatPatAux 0 p;

formatPats ps = hsep (map (formatPatAux 10) ps);

formatAlt (p, e) = formatPat p <+> text "->" <+> formatExpr e;
formatAlts alts  = vcat (punctuate (char ';') (map formatAlt alts));

formatExprAux _ (Var id)   = textIdOrOp id;
formatExprAux _ (Lit l)    = formatLiteral l;
formatExprAux _ (Const id) = textIdOrOp id;
formatExprAux d e@(Ap _ _)   =
  let {
    sepArgs (Ap f a) acc = sepArgs f (a:acc);
    sepArgs f        acc = (f, acc);
  } in
  case sepArgs e [] of {
     (Const id, args) | isTupleId id ->
             parens (hsep (punctuate (text ",") (map (formatExprAux 0) args)));
     (f, args) -> parensIf (d>=11)(formatExprAux 10 f <+> hsep (map (formatExprAux 11) args));
  };
formatExprAux d (Let bg e) =
  parensIf (d>=11) (text "let" <+>
		    braces (formatBindGroup bg) <+>
                    text "in" <+>
		    formatExprAux 10 e);
formatExprAux d (Lam (ps, e)) =
    parensIf (d>=11) (char '\\' <+>
		      formatPats ps <+>
		      text "->" <+>
		      formatExprAux 10 e);
formatExprAux d (If b e1 e2) =
    parensIf (d>=11) (text "if" <+>
		      formatExprAux 10 b <+>
		      text "then" <+>
		      formatExprAux 10 e1 <+>
		      text "else" <+>
		      formatExprAux 10 e2);
formatExprAux d (Case e alts) =
    parensIf (d>11) (text "case" <+>
		     formatExprAux 10 e <+>
		     text "of" <+>
		     braces (formatAlts alts));			   

formatExpr e = formatExprAux 0 e;
}
