-- Koji Kagawa
-- Parser, PVMain$B$G;HMQ(B
-- Kind Inference$B$r(B Type Inference$B$GBeMQ$9$k$?$a$KJQ49$r9T$&(B
--

module KindInfer where {

import Id;
import Kind;
import Type;
import Pred;
import Scheme;
import Assump;
import TIMonad;
import TIMain;
import Pat;
import List ((\\));
import Unify;
import StaticPrelude;

tStar = TCon (Tycon "*" Star);
tKfun = TCon (Tycon "(->)" (Kfun Star (Kfun Star Star)));

kind2type Star         = tStar;
kind2type (Kfun k1 k2) = TAp (TAp tKfun (kind2type k1)) (kind2type k2);

type2kind (TCon (Tycon _ Star)) = Star;
type2kind (TAp (TAp _ k1) k2)   = Kfun (type2kind k1) (type2kind k2);
type2kind (TVar _)              = Star;
type2kind _                     = error "type2kind";

type RawPred = (Id, [Id]);
type RawInst = ([RawPred], (RawPred, Id, RawPred));
type RawType = Expr;

-- Kind$B$J$I$K4X$9$kJd=u4X?t(B
--
-- Expr$BCf$N<+M3JQ?t(B 
varsExpr :: Expr -> [Id] -> [Id];
varsExpr (Var i) acc    = i:acc;
varsExpr (Ap f a) acc   = varsExpr f (varsExpr a acc);
varsExpr (Const _) acc  = acc;  			
varsExpr (If e1 e2 e3) acc  = varsExpr e1 (varsExpr e2 (varsExpr e3 acc));
varsExpr (Lam (pats, e)) acc = (varsExpr e [] \\ varsPats pats []) ++ acc;
varsExpr (Case e alts) acc = concat (map varsPE alts) ++ varsExpr e acc;
varsExpr (Let bg e) acc = let (ids, vars) = varsBg bg in ((varsExpr e vars) \\ ids) ++ acc;
varsExpr           _   acc = acc;

-- BindingGroup$BCf$N<+M3JQ?t(B
varsBg (es, is) = let { (is1, vs1) = unzip (map varsExpl es);
			(is2, vs2) = unzip (concat (map (map varsImpl) is)) }
		  in (is1++is2, concat vs1++concat vs2);

varsExpl (id, _, alts) = (id, concat (map varsAlt alts));
varsImpl (id, alts)    = (id, concat (map varsAlt alts));

varsPE :: (Pat, Expr) -> [Id];
varsPE (p, e) = varsExpr e [] \\ varsPat p [];

varsAlt :: Alt -> [Id];
varsAlt (pats, e) = varsExpr e [] \\ varsPats pats [];

varsAlts :: [Alt] -> [Id];
varsAlts alts = concat (map varsAlt alts);	    

-- Pattern$BCf$N<+M3JQ?t(B 
varsPat :: Pat -> [Id] -> [Id];
varsPat (PVar id) acc = id : acc;	       
varsPat (PAs id pat) acc =  id : varsPat pat acc;
varsPat (PCon c pats) acc = varsPats pats acc;
varsPat (PLazy pat) acc   = varsPat pat acc;
varsPat (PNpk id _) acc = id : acc;
varsPat _ acc = acc;

varsPats :: [Pat] -> [Id] -> [Id];
varsPats pats acc = foldl (flip varsPat) acc pats;

----------------------------------------------------------------------
--
-- ClassEnv$B$NCf$N(BKind$B>pJs$rMxMQ$7$F(B RawPred$BCf$NJQ?t$r=i4|2=(B
--

kindPred :: ClassEnv -> RawPred -> TI ();
kindPred ce (s, stvs) = case classes ce s of {
  Just (tvs, is, its, v, ids) ->  sequence_
	$ zipWith (\ id (Tyvar _ k) ->
	              do {
			   s  <- getSubst;
			   case lookup (Tyvar id Star) s of {
			    Nothing -> do {
			      sb <- varBind (Tyvar id Star) (kind2type k);
	       		      extSubst sb
                            };
                            Just t -> unify t (kind2type k)
			   }
	       		 }) stvs tvs;
  Nothing -> return ()
}
;

kindPreds :: ClassEnv -> [RawPred] -> TI ();
kindPreds ce sc =
    do {
	mapM (kindPred ce) sc;
	return ()
	}
;		  

-- kind inrefence$B$rDL>o$N(Btype inference$B$GBeMQ$9$k(B
constKinds = [("(->)",  tArrow), ("Either", tEither), ("Maybe", tMaybe), ("Ordering", tOrdering),
	      ("()", tUnit), ("(,)", tTuple2), ("(,,)", tTuple3), ("(,,,)", tTuple4), ("(,,,,)", tTuple5),
	      ("(,,,,,)", tTuple6), ("(,,,,,,)", tTuple7),
              ("Int", tInt), ("Double", tDouble)				    
	     ];


initAssump = map (\ (id, t) -> id :>:  toScheme (kind2type (kind t))) constKinds;

kindExp :: ClassEnv -> ([RawPred], RawType) -> TI ();
kindExp ce (c, e) = do {
			 kindPreds ce c;
			 let { vs = varsExpr e [] };
			 acc <- mapM (\ i -> do {
			   s <- getSubst;
			   -- $BJQ?t(Bi$B$KF1$8$J$^$($N7?JQ?t(Bi$B$r3d$jEv$F$k(B 
 	                   return (i :>: toScheme (TVar (Tyvar i Star)))
			 }) vs;
			 ([], (t, _, _)) <- tiExpr initialEnv (acc++initAssump) e;
			 unify t (kind2type Star); -- RawType$B$O(B Kind *$B$H2>Dj(B
			}
;

kindExps ce es = mapM (kindExp ce) es;

kindInst :: ClassEnv -> RawInst -> TI ();
kindInst ce (ps, ((v, vs), x, (r, rs)))  = do { 
         kindPreds ce ps;
	 kindPred ce (v, x:vs);
	 kindPred ce (r, x:rs);
};

----------------------------------------------------------------------
-- Raw$B7A<0$+$i(BKind$BIU$-$N7A<0$X$NJQ49(B
--

myfind p ((x,y):xs) = if p == x then y else myfind p xs;
myfind p []          = error ("myfind:" ++ show p ++ " not found.");


findTyvar i ((tv @ (Tyvar j k)) : vs) = if i==j then tv else findTyvar i vs;

-- RawType (Expr)$B$+$i(B Type$B$XJQ49(B 
kindReplaceType :: [Tyvar] -> RawType -> Type;
kindReplaceType vs (Var id)   = TVar (findTyvar id vs);
kindReplaceType vs (Ap t1 t2) = TAp (kindReplaceType vs t1) (kindReplaceType vs t2);
kindReplaceType vs (Const c) =  myfind c constKinds;

-- RawPred $B$+$i(B Pred$B$XJQ49(B
kindReplacePred :: [Tyvar] -> RawPred -> Pred;
kindReplacePred vs (c, targs) = IsIn c (map (kindReplaceType vs . Var) targs);

kindReplaceQual :: [Tyvar] -> ([RawPred], RawType) -> Qual Type;
kindReplaceQual vs (c, t) = map (kindReplacePred vs) c :=> kindReplaceType vs t;

kindReplaceInst :: [Tyvar] -> RawInst -> Inst;
kindReplaceInst vs (ps, (v, x, r)) =
    map (kindReplacePred vs) ps
	    :=> (kindReplacePred vs v, findTyvar x vs, kindReplacePred vs r);
}
