-- Koji Kagawa
-- main function for Polymorphic Variant transformation
module PVMain where { 

import Maybe (fromJust);

import Text.ParserCombinators.Parsec;

import PPrint (render);
import Id;
import Kind;
import Type;
import Subst;
import Unify;
import Pred;
import Scheme;
import Assump;
import Pat;
import TIMonad;
import TIMain;
import KindInfer;
import Scc;
import List (nub, (\\), intersect, partition, delete);
import StaticPrelude;
import Parser;
import Format;

-- import Hugs.Observe;

----------------------------------------------------------------------
-- classify TopDecls
 
splitTopDecls :: [RawTopDecl] -> ([RawTopDecl], [Decl], [RawTopDecl]);
splitTopDecls decls = splitTopDecls0 decls ([], [], []);

splitTopDecls0 (RawTopDecl d : ts) (vrs, ds, is)        = splitTopDecls0 ts (vrs, d:ds, is);
splitTopDecls0 (i@(RawInstDecl _ _) : ts) (vrs, ds, is) = splitTopDecls0 ts (vrs, ds, i:is);
splitTopDecls0 (vr:ts) (vrs, ds, is)                    = splitTopDecls0 ts (vr:vrs, ds, is);
splitTopDecls0 [] (vrs, ds, is) = (vrs, ds, is);

----------------------------------------------------------------------
-- dependency among Tycon, Class
--

myfindp p (x:xs) = if p x then x else myfindp p xs;
myfindp p []     = error ("myfindp: error"); 

depTyconsEDecls es = concat (map depTyconsEDecl es);

depTyconsEDecl (GenDecl _ (ps, t)) = depTyconsPreds ps ++ depTyconsType t;
depTyconsEDecl _                   = [];

depTyconsPreds ps = concat (map depTyconsPred ps);

depTyconsPred (id, ts) = [id];

depTyconsTypes ts = concat (map depTyconsType ts);

depTyconsType t = depTyconsTypeAux t [];

depTyconsTypeAux (Const id) acc = id:acc;
depTyconsTypeAux (Ap t1 t2) acc = depTyconsTypeAux t1 (depTyconsTypeAux t2 acc);
depTyconsTypeAux _          acc = acc;                                    

depTyconsRawTopDecl :: RawTopDecl -> (Id, [Id]);
depTyconsRawTopDecl (RawVariantDecl sc p es) = depTyconsRawTopDeclAux sc p es;
depTyconsRawTopDecl (RawRecordDecl sc p es)  = depTyconsRawTopDeclAux sc p es;

depTyconsRawTopDeclAux :: [RawPred] -> RawPred -> [Decl] -> (Id, [Id]);
depTyconsRawTopDeclAux sc (id, _) es = (id, map fst sc ++ depTyconsEDecls es);

-- devide TopDecl into scc's
tdeclsDeps :: [RawTopDecl] -> [[RawTopDecl]];
tdeclsDeps ts = let { ds  = map depTyconsRawTopDecl ts;
                      ids = map fst ds;
                      xs  = map (\ (id, deps) ->
                                 (id, Left (nub (filter (\ a -> a `elem` ids) deps))))
                                ds; 
                      scs = scc ids xs;
                      declId (RawVariantDecl _ (id, _) _) = id;
                      declId (RawRecordDecl  _ (id, _) _) = id
                    }
                 in  map (map (\ id-> myfindp (\ d -> declId d == id) ts)) scs;   



----------------------------------------------------------------------
-- kind inference relation

kindTopDecl :: ClassEnv -> RawTopDecl -> TI ();
kindTopDecl ce (RawVariantDecl sc p cdecls) =
    do {
        kindPreds ce sc;
        kindExps ce (map (\ (GenDecl id ct) -> ct) cdecls);
        return ()
       }
;
kindTopDecl ce (RawRecordDecl sc p cdecls) =
    do {
        kindPreds ce sc;
        kindExps ce (map (\ (GenDecl id ct) -> ct) cdecls);
        return ()
       }
;

inferClassKindAux2 ce s sc (c, ids) cdecls flag =
      let {
          vs =  map (\ i -> Tyvar i (type2kind (apply s (TVar (Tyvar i Star))))) ids;
          sc1 = map (kindReplacePred vs) sc;

          idDecls (GenDecl id _) = Just id;
          idDecls _              = Nothing;

          maybeConcat (Just x:xs) = x:maybeConcat xs;
          maybeConcat (_:xs)      = maybeConcat xs;
          maybeConcat []          = [];                   

          newCe = modify ce c (vs, sc1, [{- instances -}], flag, maybeConcat (map idDecls cdecls))
      } in newCe
;

inferClassKindAux ce s (RawVariantDecl sc (c, ids) cdecls) =
      inferClassKindAux2 ce s sc (c, ids) cdecls True;

inferClassKindAux ce s (RawRecordDecl sc (c, ids) cdecls) =
      inferClassKindAux2 ce s sc (c, ids) cdecls False;

addHeadCDecl p (GenDecl id (ps, t)) = GenDecl id (p:ps, t);
addHeadCDecl p d = d;

-- add constructors and selectors to Assump
addMember :: ClassEnv -> RawTopDecl -> [Assump];
addMember ce (RawVariantDecl sc (c, ids) cdecls) =
    let { cdecls1 = map (addHeadCDecl (c, ids)) cdecls;
          (es, [{- Impl-part should be empty -}]) = decls2Bg (Right ce) cdecls1 } in
    map (\ (id, sc, [{- empty -}]) -> id :>: sc) es;      
addMember ce (RawRecordDecl sc (c, ids) cdecls) =
    let { cdecls1 = map (addHeadCDecl (c, ids)) cdecls;
          (es, [{- Impl-part should be empty -}]) = decls2Bg (Right ce) cdecls1 } in
    map (\ (id, sc, [{- empty -}]) -> id :>: sc) es;      


-- Infer Kinds for raw ClassDecl
inferClassKind :: ClassEnv -> [Assump] -> [RawTopDecl] -> (ClassEnv, [Assump]);
inferClassKind ce ass tds =
   runTI $
   do {
        mapM (kindTopDecl ce) tds;
        s <- getSubst;
        let { newCe  = foldl (\ ce td -> inferClassKindAux ce s td) ce tds;
              newAss = map (\ td -> addMember newCe td) tds } in
        return (newCe, concat newAss ++ ass)
      }
;

----------------------------------------------------------------------
-- infer Kinds for RawInst

checkInst :: ClassEnv -> RawInst -> ClassEnv;
checkInst ce (ps, h) =       
             runTI $
             do  {
                kindInst ce (ps, h);
                s <- getSubst;
                let { vs = map (\ (Tyvar id k, t) ->
                                   Tyvar id (type2kind (apply s (TVar (Tyvar id k))))) s;
                      (qs:=>p) = kindReplaceInst vs (ps, h);
                      Just ce1 = addInst qs p ce;
                    };
                return ce1
             }
;

----------------------------------------------------------------------
-- reduce instance dalcarations into explicitly typed declarations 

attachScheme v r ce as (id, _{- should be Nothing -}, alts)  =
   let {
      t = runTI $ do {
                         sc <- find id as;
                         ps:=>t1 <- freshInst sc;
                         let { Just (qs :=> (IsIn v0 vs, x, IsIn r0 rs)) = findInst ce (v, r);
                               xx = TVar x
                             };
                         a       <- newTVar Star;
                         unify t1 (xx `fn` a);
                         s0 <- getSubst;
                         let { ps1 = IsIn v0 (xx:vs) : IsIn r0 (xx:rs) : qs++ps;
                               (s, ps2) = simplify ce (apply s0 ps1);
                               t2 = apply s0 t1;
                               t3 = apply s t2;
                               ret = ps2:=>t3
                             } in
                         
                         return (quantify (tv ret) ret) };
   } in
   (id, Just t, alts);
;

----------------------------------------------------------------------
-- process Ambiguity
--
    

enumConstructorTypes ce ass m vs = do {
  let {
      -- type of each method
      constrType ce ass p@(IsIn id _) rt mid = do {
          sc         <- find mid ass;                        
          (ps :=> t) <- freshInst sc;
          let {
            (_, rt1) = domainTypes t [];
            p1       = myfindp (\ (IsIn id1 _) -> id1==id) ps;
          };
          s0 <- match rt1 rt;  
          s1 <- match p1 p;   -- if this fails, we need GADT (ToDo).
          let {
                s  = s1 @@ s0;
                t1 = apply s t;
                qs = apply s ps;
                qs1 = filter (\ (IsIn i _) -> i /= id) qs
              };
          return (qs1 :=> t1)
        }
      ;

      constrTypes ce ass t0 p@(IsIn id _) = do {
          let {
            mids  = methods ce id;
          };
          types <- mapM (constrType ce ass p t0) mids;
          return types;
      }
      ;  

      domainKinds :: Kind -> [Kind] -> [Kind]; -- the list of argument types of a function type
      domainKinds (Kfun a b) ks = domainKinds b (a:ks);
      domainKinds Star       ks = reverse ks;

      tapList t [] = t;
      tapList t (x:xs) = tapList (TAp t x) xs;

      kinds = domainKinds (kind m) [];
  };
  ts <- mapM newTVar kinds;
  let { t0 = tapList m ts };  	
  css <- mapM (constrTypes ce ass t0) vs;			     
  return (t0, css);
};


replaceTopPat id (PCon c ps) = PCon ('C':theStr++c++theStr++id) ps;
replaceTopPat id (PAs v p)   = PAs v (replaceTopPat id p);
replaceTopPat id p           = p;

replaceTopAlt id (p:pats, e) = (replaceTopPat id p : pats, e);

replaceTop id (f, _, alts) = (f, map (replaceTopAlt id) alts);

type InnerConstDecl = (Id, ([Tyvar]{-forall-}, [Type]));
type InnerDataDecl  = (Id, [Tyvar], Maybe (Qual [InnerConstDecl]), [(Id, Qual Type)]);
type InnerInstDecl  = (Qual Pred, [Impl]);

-- generate an instance declaration in traditional format
ofinst :: Inst -> (Pred, Type, Pred) -> [Pred];
ofinst (ps:=>(v0, tv, r0)) (IsIn v (_:vargs), t, IsIn r (_:rargs))  =
    let {
         Just s =  match [v0, r0] [IsIn v vargs, IsIn r rargs];
         Just s1 = merge (tv +-> t) s;
         qs = apply s1 ps;
     } in qs;

domainTypes :: Type -> [Type] -> ([Type], Type); -- the list of argument types of a function type
domainTypes (TAp (TAp (TCon (Tycon "(->)" _)) a) b) ts = domainTypes b (a:ts);
domainTypes t                                       ts = (reverse ts, t);


fapp (TAp f a) args = fapp f (a:args);
fapp  t        args = (t, args);

extv w t@(TAp t1 t2) acc  --  remove a and b when t is in the form: m a b.
      = let { (f, targs) = fapp t [] } in
        if f == TVar w then tv targs ++ acc else (extv w t1 (extv w t2 acc)); 
extv w _ acc = acc;


-- generate data declaration + instance declaration from FreezedAmbiguity (= (Kind, [Id])) 
-- ToDo: generate GADT
constructDeclsFromAmbiguity
  :: Id -> ClassEnv -> [InstDefn] -> [Assump] -> FreezedAmbiguity -> (Type, InnerDataDecl, [InnerInstDecl]);
constructDeclsFromAmbiguity id ce ins ass (kindv, classNames) =
    let {
	 -- regenerate v and ps
         (v, _, vs, rs, rt, cs) =
               runTI $ do {
                   v0@(TVar tv0)  <- newTVar kindv;
	           ps0 <- mapM (\ i -> do {
                                   let { Just (_:tvs, _, _, _, _) = classes ce i };
                                   ts1 <- mapM (\ t0 -> newTVar (kind t0)) tvs;
                                   return (IsIn i (v0:ts1))
                               }) classNames;  --- preds with fresh parameters
                   let { (_, ps1) = simplifyA ce ps0; };
                   (_, ps2) <- checkPreds ce ps1;
		   let { (_, ps3) = simplifyA ce ps2; }; -- restore super classes
		   -- calculate cs here
                   let { (vs, rs) = partition (\ (IsIn id _) -> variantPred ce id)
			                      ps3; };
                   (rt, css) <- enumConstructorTypes ce ass v0 vs;
                   let { cs = concat css; };  -- the types of constructors --! [(a -> m a), (m b -> (b -> m a) -> m a)]
                   return (tv0, ps3, vs, rs, rt, cs);
               };
         
         cnames = concat (map (\ (IsIn id _) -> methods ce id) vs); -- the names of constructors
         args = let { -- those type variables that should be type parameters in constructor types -- ! []
                  mytv (qs:=>t) = tv (qs:=>t) \\ extv v t [];   -- ! [m] -- remove a and b
                } in  delete v (nub (concat (map mytv cs)));
	 args2 = let {  -- those type variables that appears in the return type ... -- ! [a]
                   (_, args2) = fapp rt [];    -- parameters of the return type
                 } in map (\ (TVar tv0) -> tv0) args2; 
         kinds = map kind args;  -- ! []
         vkind = kind v; -- ! Star -> Star
         k  = foldr Kfun vkind kinds;  -- the kind of the type constructor
         t0 = foldl (\ t v -> TAp t v) (TCon (Tycon ("Data"++theStr++id) k)) (map TVar args);
                                                   -- (part of) the return type that substitutes v
         dataDecl = let {
                      addIds     = zip (map (\ i -> 'C':theStr++i++theStr++id) cnames);                      
                    } in 
                    case 
                      let {
         		-- substitute the self type
                        mapMaybe f [] = Just [];
                        mapMaybe f (x:xs) = case (f x, mapMaybe f xs) of {
                                              (Just y, Just ys) -> Just (y:ys);
                                              _                 -> Nothing   
                                            };
         		applySubst (qs:=>t) = let {
         		    (ts, rt)   = domainTypes t [];
         		    rargs = mapMaybe (\ x -> case x of { TVar tv0 -> Just tv0; _ -> Nothing })
                                             (snd (fapp rt []));  -- fail -> GADT
                          } in
                          case rargs of {
                            Just rs -> Just (qs, (rs, 
         		        concat (map (\ t ->  extv v t []) ts) \\ rs,
         		         -- the type parameters that should be quantified (forall) -- ! [b]
         		        map (apply [(v, t0)]) ts));
                            Nothing -> Nothing
                          };
                        applySubst2Aux rs0 (qs, (rs1, ex, ts)) =
                          let { sub = zip rs1 (map TVar rs0) } in
                          (map (apply sub) qs, (ex, map (apply sub) ts));
                        applySubst2 (Just ((qs, (rs0, ex, ts)) : xs)) =
                          Just ((qs, (ex, ts)) : map (applySubst2Aux rs0) xs);
                        applySubst2 Nothing = Nothing
                      } in applySubst2 (mapMaybe applySubst cs)
                of {
                   Just qtss ->
                     let {
                       (qss, tss) = unzip qtss;
                       (_sub, qs) = simplify ce (concat qss);
                       constrDefs = addIds tss;
                     } in
                      ("Data"++theStr++id, args ++ args2, 
                                           Just (qs :=> constrDefs), addIds (map (apply [(v, t0)]) cs));
                   _ -> ("Data"++theStr++id, args ++ args2, Nothing, addIds (map (apply [(v, t0)]) cs));
                };

         variantInsts = map (\ vp@(IsIn vid (x:vargs)) ->
			     let { cxs = methods ce vid;
				   cdefs = map (\ i -> (i, [([], Var ('C':theStr++i++theStr++id))])) cxs
			     } in
	 		     ([] :=> IsIn vid (apply [(v, t0)] x:vargs), cdefs)
			    ) vs ;

         recordInsts = map (\ rp@(IsIn rid (x:rargs)) ->
			    let { 
                                  qss = map (\ vp@(IsIn vid _) -> ofinst (fromJust (findInst ce (vid, rid)))
					                               (vp, t0, rp)) vs;
				  defss = map (\ (IsIn vid _) -> map (replaceTop id) (myfind (vid, rid) ins)) vs
                                }
			    in (concat qss :=> IsIn rid (apply [(v, t0)] x:rargs), concat defss)) rs
			    -- collect instance definitions for each record pred

    } in 
   (t0, dataDecl, recordInsts++variantInsts);

-- from ExtAmbiguity to data definitions and instance definitions
processExtAmbiguity :: ClassEnv -> [InstDefn] -> [Assump] -> ExtAmbiguity
                                -> ([InnerDataDecl], [InnerInstDecl], Expl);
processExtAmbiguity ce ins ass (ambs, t, id) =
    let {
        ids = map (\ s -> id++s) ("" : map (\ i -> "_"++show i) [1..]); -- id, id_1, id_2, ...
        dts = map (\ (id, amb) -> constructDeclsFromAmbiguity id ce ins ass amb) (zip ids ambs);
        (tdefs, idefs) = unzip (map (\ (a, b, c) -> (b, c)) dts); 

        Forall ks (_:=>t0) = t;

        len = length ks - length ambs;  
        ts = map TGen [0..len-1] ++ map fst3 dts;
        t1 = inst ts t0;        
        sc = Forall (take len ks) ([] :=> (t1 `fn` t1));
        expl = ("id"++theStr++id, sc, [([PVar "x"], Var "x")]);
    }   
    in (tdefs, concat idefs, expl)
;

----------------------------------------------------------------------
-- main part


initState :: (ClassEnv, [InstDefn], [Assump]);
initState = (initialEnv,
             [],
	     ["." :>: Forall [Star, Star, Star] ([] :=>
		       ((TGen 0 `fn` TGen 1) `fn` (TGen 2 `fn` TGen 0) `fn` TGen 2 `fn` TGen 1)),
	      falseCfun, trueCfun,
	      nothingCfun, justCfun, leftCfun, rightCfun,
	      eqMfun, neqMfun, eqDMfun, neqDMfun, 
	      compareMfun, ltMfun, leMfun, gtMfun, geMfun, ltDMfun, leDMfun, gtDMfun, geDMfun,
	      maxMfun, minMfun,
              unitCfun, tup2Cfun, tup3Cfun, tup4Cfun, tup5Cfun, tup6Cfun, tup7Cfun, 
              plusMfun, minusMfun, timesMfun, plusDMfun, minusDMfun, timesDMfun, negateMfun, absMfun, signumMfun,
	      quotMfun, remMfun, divMfun, modMfun, quotRemMfun, divModMfun,
	      evenMfun, oddMfun,
	      divideMfun, recipMfun,
	      piMfun, expMfun, logMfun, sqrtMfun, starstarMfun, logBaseMfun, sinMfun, cosMfun, tanMfun, 
              asinMfun, acosMfun, atanMfun, sinhMfun, coshMfun, tanhMfun, asinhMfun, acoshMfun, atanhMfun,
	      properFractionMfun, truncateMfun, roundMfun, ceilingMfun, floorMfun
             ]);

collectConstrDecl (GenDecl id _ : rest) acc = collectConstrDecl rest (id:acc);
collectConstrDecl (_ : rest) acc            = collectConstrDecl rest acc;
collectConstrDecl [] acc                    = acc;
					      
collectConstrAux [] acc = acc;
collectConstrAux (RawVariantDecl _ _ ds : rest) acc =
  collectConstrAux rest (collectConstrDecl ds acc);  
collectConstrAux (_ : rest) acc = collectConstrAux rest acc;    

collectConstr rtds = collectConstrAux rtds [];

replaceConstrId names id = {- if id `elem` names then "_"++theStr++id else -} id;
replaceConstrAlt names (ps, e) = (ps, replaceConstrExpr names e);
replaceConstrImpl names (id, alts)     = (replaceConstrId names id, map (replaceConstrAlt names) alts);
replaceConstrExpl names (id, sc, alts) = (replaceConstrId names id, sc, map (replaceConstrAlt names) alts);
replaceConstrBindGroup names (es, iss) = (map (replaceConstrExpl names) es, map (map (replaceConstrImpl names)) iss);
replaceConstrDecl names (Decl id alt)  = Decl (replaceConstrId names id) (replaceConstrAlt names alt);
replaceConstrDecl names (GenDecl id t) = GenDecl (replaceConstrId names id) t;
replaceConstrDecl names d              = d; 

replaceConstrExpr names e@(Const id)  = if id `elem` names then Var ("_"++theStr++id) else e;
replaceConstrExpr names (Ap e1 e2) = Ap (replaceConstrExpr names e1) (replaceConstrExpr names e2);
replaceConstrExpr names (Let bg e) = Let (replaceConstrBindGroup names bg) (replaceConstrExpr names e);
replaceConstrExpr names (Lam alt)  = Lam (replaceConstrAlt names alt); 
replaceConstrExpr names (If e1 e2 e3) = If (replaceConstrExpr names e1) (replaceConstrExpr names e2) (replaceConstrExpr names e3);
replaceConstrExpr names (Case e alts) = Case (replaceConstrExpr names e) (map (\ (p, e1) -> (p, replaceConstrExpr names e1)) alts); 
replaceConstrExpr names e = e;

replaceConstrRawTopDecl names (RawVariantDecl ps p decls) = RawVariantDecl ps p (map (replaceConstrDecl names) decls);
replaceConstrRawTopDecl names d = d;

-- main function
runAux2 tdecls ce0 ass instD = let {
  (vrs, funs, insts) = splitTopDecls tdecls;
  tdss = reverse (tdeclsDeps vrs);
  cstrs = collectConstr (concat tdss); -- used for substitution

  -- add class definitions
  (ce1, ass1) = foldl (\ (ce, ass) tds -> inferClassKind ce ass tds) (ce0, ass) tdss; 
      
  -- add instances (in the variant side)
  ce = foldl (\ ce (RawInstDecl i ds) -> checkInst ce i) ce1 insts; 

  -- add function definitions
  fdecls = declsCollect (Right ce) funs;

  -- devide function definitions into scc's
  ids = map (\ (id, _, _) -> id)  fdecls;
  fdeps  = map (\ (id, _, alts) -> (id, Left (intersect ids (varsAlts alts)))) fdecls;
  scs    = scc ids fdeps;

  myfind3 a ((x, y, z):xs) = if a==x then (x, y, z) else myfind3 a xs;
  myfind3 a []             = error (show a ++ " not found.");

  fdss   = map (map (\ id -> myfind3 id fdecls)) scs;
  bgs    = map toBg fdss;

  -- type-check function definitions
  loop ce as ambs bgs' (bg:bgs) = 
      do {
          (ps, (as1, amb, bg')) <- tiBindGroup ce as bg;
          (pss, as9, amb9, bgs9) <- loop ce (as1++as) (amb++ambs) (bg':bgs') bgs;
          return (ps:pss, as9, amb9, bgs9)
         };
  loop ce as ambs bgs' [] = return ([], as, ambs, bgs');
  (_pss, ass9, ambInfo, bindGroups) = runTI $ loop ce ass1 [] [] bgs;

  -- type-check instance definitions
  (instDecls, instD2) = 
	foldr (\ (RawInstDecl inst decls) (acc1, acc2) -> 
            let {
              bg  = declsCollect (Right ce) decls;
              (_, ((v, _), _, (r, _))) = inst;
              bg1 = map (attachScheme v r ce ass9) bg;
            } in
            (bg1:acc1, ((v, r), bg1):acc2)
          ) ([], instD) insts; 
   _ = runTI $ tiBindGroup ce ass9 (toBg (concat instDecls)); -- check only

   --finally, generate data declarations and instance declarations
  (dataDecls, instDecls2, castDefns) = unzip3 (map (processExtAmbiguity ce instD2 ass9) ambInfo)
}
  in (tdecls, -- return the original
      ass9,
      map (replaceConstrRawTopDecl cstrs) (concat tdss),
      map (replaceConstrBindGroup cstrs) bindGroups,
      concat dataDecls,
      map (\ (p, impls) -> (p, map (replaceConstrImpl cstrs) impls)) (concat instDecls2),
      castDefns, ce);

runAux 
  = do { whiteSpace;
         tdecls <- parseRawTopDecls;
         ce <- getClassEnv;
         ass <- getAssump;
         insts <- getInstDefn;
         return (tdecls, ce, ass, insts); 
    } 
;

formatProgram topDecls typeDecls dataDecls instDecls castDecls bindGroups = do {
       putStr "-- Original Top Declarations\n";
       putStr "{-\n"; 
       putStr $ render $ formatClassDecls topDecls;
       putStr "\n-}\n\n"; 
       putStr "-- Class Declarations\n";
       putStr $ render $ formatClassDecls typeDecls;
       putStr "\n\n";
       putStr "-- Data Declarations\n";
       putStr $ render $ formatInnerDataDecls dataDecls;                 
       putStr "\n\n";
       putStr "-- Instance Declarations\n";
       putStr $ render $ formatInnerInstDecls instDecls;
       putStr "\n\n";
       putStr "-- Cast(?) Declarations\n";
       putStr $ render$ formatExpls castDecls; 
       putStr "\n\n";
       putStr "-- BindGroups\n";
       putStr $ render $ formatBindGroups bindGroups;	     
 }
;

run :: String -> IO ();
run file = 
  do { input <- readFile file;
       (tdecls, ce, ass, instD) <- 
         case runParser runAux initState file input of {
           Left err -> fail (show err);
           Right z  -> return z
         };
       let { (topDecls, ass1, typeDecls, bindGroups, dataDecls, instDecls, castDecls, ce1)  =
             	runAux2 tdecls ce ass instD };
       formatProgram topDecls typeDecls dataDecls instDecls castDecls bindGroups;
       return ();
     };
}
