{-
 Monad$B%/%i%9$r;HMQ$7$J$$%P!<%8%g%s(B
 $B;29M(B
   Monad Transformers and Modular Interpreters 
      Sheng Liang, Paul Hudak, Mark Jones   
   Haskell$B$N(BPrelude.hs
   Monads for Functional Programming
      Philip Wadler
-}
module ParserMonad where {

type Parser d a =  d -> [(a, d)];

unitP :: a -> Parser d a;
unitP a = \ str -> [(a, str)];

bindP :: Parser d a -> (a -> Parser d b) -> Parser d b;
m `bindP` k = \ str -> concat (map (uncurry k) (m str));

bind_P :: Parser d a -> Parser d b -> Parser d b;
m `bind_P` n = m `bindP` \ _ -> n;

-- $BHs7hDjE*A*Br(B
mplusP :: Parser d a -> Parser d a -> Parser d a;
m1 `mplusP` m2 = \ str -> m1 str ++ m2 str;

-- $B%P%$%"%9$D$-A*Br(B
choiceP :: Parser d a -> Parser d a -> Parser d a;
m1 `choiceP` m2 = \ str -> let ds = m1 str in
                           case ds of {
                             [] -> m2 str;
                             _  -> ds
                           };

infixr 1 `choiceP`;

-- $B<:GT(B
mzeroP :: Parser d a;
mzeroP = \ str -> [];

sat :: (a -> Bool) -> Parser [a] a;
sat p (h:t) = if p h then [(h, t)] else [];
sat p []    = [];

-- 0$B2s0J>e$N7+JV$7(B  
many :: Parser d a -> Parser d [a];
many m  = let q = (m `bindP` \ x -> q `bindP` \ xs -> unitP (x:xs))
                  `choiceP`
                  unitP [] 
          in q ;

-- 1$B2s0J>e$N7+JV$7(B  
many1 :: Parser d a -> Parser d [a];
many1 m = m `bindP` \ x -> many m `bindP` \ xs -> unitP (x:xs);


{----------------------------------------------------------------------
    $B;z6g2r@OIt(B
 ----------------------------------------------------------------------}

data Token = Number Double | Ident String | Symbol String | Single Char
           | Keyword String  
	     deriving (Show, Eq);

-- identTail = [a-zA-Z0-9_']*
identTail :: Parser String String;
identTail = many (sat (\ c -> isAlpha c || isDigit c || c=='_'  || c=='\''));

-- ident = [a-zA-Z_][a-zA-Z0-9_']*
ident :: Parser String String;
ident = sat (\ c -> isAlpha c || c=='_') `bindP` \ x ->
        identTail `bindP` \ xs ->
        unitP (x:xs);

strToInt :: String -> Int;
strToInt cs =
  let {
    foo [] acc     = acc;
    foo (c:cs) acc = foo cs (acc*10+(ord c - ord '0')) 
  } in  foo cs 0;

strToDoubleTail :: String -> Double ;
strToDoubleTail cs =
  let {
    foo []     mag = 0.0;
    foo (c:cs) mag = (fromInt (ord c - ord '0'))*mag + foo cs (mag*0.1)
  } in 
  case cs of {
     "" -> 0.0;
     '.':ds -> foo ds 0.1; 
     _  -> {- error -} 0.0
  };

-- number = [0-9]*(\.[0-9]*)?
digits :: Parser String (String, String);
digits = many (sat isDigit) `bindP` \ x ->
         (
           (sat (\ c -> c=='.') `bindP` \ c ->
            many (sat isDigit) `bindP`  \ y ->
            unitP (c:y))
           `choiceP`
            unitP ""
         ) `bindP` \ y ->
         unitP (x, y) ;

-- $B&E$K%^%C%A$J$i$P(B0$B$rJV$9(B
number :: Parser String Double;
number = digits `bindP` \ (x, y) ->
         unitP (fromInt (strToInt x) + strToDoubleTail y);

-- $B&E$K%^%C%A$J$i$P(B0$B$rJV$5$:$K<:GT$9$k(B
number1 :: Parser String Double;
number1 = digits `bindP` \ (x, y) ->
          if (x++y)=="" then
             mzeroP
          else
             unitP (fromInt (strToInt x) + strToDoubleTail y);

-- single = [,;()\[\]{}_`]
single :: Parser String Char;
single = sat (\ c -> c `elem` ",;()[]{}_`");

-- symbol = [!@#$%&*+./<=>?\\^|:-~]+
symbol :: Parser String String ;
symbol = many1 (sat (\ c -> c `elem` "!@#$%&*+./<=>?\\^|:-~"));

-- mylex : $B;z6g2r@OItK\BN(B
-- ToDo $B%3%a%s%H$N=hM}(B
mylex :: [String] -> Parser String [Token];
mylex kwd =(
       	      (
       	   	 (sat isSpace `bindP` \ _ -> unitP [])
       	   	 `choiceP` 
       	   	 (number1 `bindP` \ d -> unitP [Number d]) 
       	   	 `choiceP` 
       	   	 (ident `bindP` \ id -> 
                  if id `elem` kwd then unitP [Keyword id]
                                   else unitP [Ident id]) 
       	   	 `choiceP` 
       	   	 (symbol `bindP` \ id -> 
                  if id `elem` kwd then unitP [Keyword id] 
                                   else unitP [Symbol id]) 
       	   	 `choiceP` 
       	   	 (single `bindP` \ c -> unitP [Single c]) 
       	      ) `bindP` \ xs ->
       	      mylex kwd `bindP` \ ys ->
       	      unitP (xs++ys)
       	    ) 
       	    `choiceP`
       	    unitP [];

check :: (Token->Bool) -> Parser [Token] Token;
check p (t:ts) = if p t then [(t, ts)] else [];
check p []     = [];


-- lexer$B$H(Bparser$B$N9g@.(B
mkFE :: Parser a b -> Parser b c -> a -> c;
mkFE lexer parser str = let { tmp = lexer str } in
                        case concat (map (\ (toks, _) -> parser toks) tmp) of {
                          (ret, rest):_ -> ret;
                          []            -> error "Syntax Error!"
                        }
}


