-----------------------------------------------------------------------------
-- Pat:		Patterns
-- 
-- Part of `Typing Haskell in Haskell', version of November 23, 2000
-- Copyright (c) Mark P Jones and the Oregon Graduate Institute
-- of Science and Technology, 1999-2000
-- 
-- This program is distributed as Free Software under the terms
-- in the file "License" that is included in the distribution
-- of this software, copies of which may be obtained from:
--             http://www.cse.ogi.edu/~mpj/thih/
-- 
-----------------------------------------------------------------------------

module Pat where
import Id
import Kind
import Type
import Pred
import Scheme
import Assump
import TIMonad
import Infer
import Lit

data Pat        = PVar Id
                | PWildcard
                | PAs  Id Pat
                | PLit Literal
                | PNpk Id Integer
                | PCon Id [Pat] -- No Type Assump in Id
                | PLazy Pat     
		  deriving Show -- added by kagawa
tiPat :: Infer Pat ([Assump], Type)
-- i.e.:: ClassEnv -> [Assump] -> Pat -> TI ([Pred], ([Assump], Type))
-- added ClassEnv and [Assump] as parameters

tiPat ce as (PVar i) = do v <- newTVar Star
                          return ([], ([i :>: toScheme v], v))

tiPat ce as PWildcard   = do v <- newTVar Star
                             return ([], ([], v))

tiPat ce as (PAs i pat) = do (ps, (as1, t)) <- tiPat ce as pat
                             return (ps, ((i:>:toScheme t):as1, t))

tiPat ce as (PLit l) = do (ps, t) <- tiLit l
                          return (ps, ([], t))

tiPat ce as (PNpk i k)  = do t <- newTVar Star
                             return ([IsIn "Integral" [t]], ([i:>:toScheme t], t))

tiPat ce as (PCon i pats) = do (ps,(as1,ts)) <- tiPats ce as pats
			       sc <- find i as 	     
                               t'         <- newTVar Star
                               (qs :=> t) <- freshInst sc
                               unify t (foldr fn t' ts)
                               return (ps++qs, (as1, t'))

tiPat ce as (PLazy pat) = tiPat ce as pat

tiPats :: Infer [Pat] ([Assump], [Type])
-- tiPats     :: ClassEnv -> [Assump] -> [Pat] -> TI ([Pred], ([Assump], [Type]))
tiPats ce as pats = do psasts <- mapM (tiPat ce as) pats
                       let ps = concat [ ps' | (ps',(_,_)) <- psasts ]
                           as = concat [ as' | (_,(as',_)) <- psasts ]
                           ts = [ t | (_,(_,t)) <- psasts ]
                       return (ps, (as, ts))

-----------------------------------------------------------------------------
