{-
   strongly connected component
   form Hugs:scc.c
   kagawa

   used in PVMain
 -}

module Scc where {

data SccState a = SccState {
		    low::     Int,
		    nodes::   [(a, Either [a] Int)],
		    daCount:: Int,
		    stack::   [a],
		    daSccs::  [[a]]
		  }
;

newSccState n = SccState {low=0, nodes=n, daCount=1, stack=[], daSccs=[]};

modifyScc ((z,w):ws) v x = if z==v then (z,x):ws
		  	           else (z,w):modifyScc ws v x;

sccMin x y = if x<=y || y==0 then x else y;

visited w nodes = case lookup w nodes of {
    Just (Left _)  -> False;
    Just (Right _) -> True;
    Nothing        -> error "visited"
};

lowlink v s0 =
    let {
	 nd = nodes s0; 
         da = daCount s0;                -- depth first search no. of v  
	 Just (Left ws) = lookup v nd;  -- adjacency list for v
         s1 = s0 { low     = da,
		   nodes   = modifyScc nd v (Right da),
		   daCount = da + 1,
		   stack   = v : (stack s0) -- push v onto stack
		 };
	 loop (w:ws) s =                 -- scan adjacency list for v
          case lookup w (nodes s) of {
             Just (Left _) -> {- not yet visited -} let {
               s1 = lowlink w s;
             } in  
             loop ws (s1{low = sccMin (low s) (low s1)});
	     Just (Right i)  -> loop ws (s {low = sccMin (low s) i});
             Nothing         -> error "lowlink:loop"
           }
         ;
	 loop []     s = s;

         s2 = loop ws s1 
     } in
     if low s2 == da then              -- start a new scc? 
       let {
         loop s acc = 
           let {
	     t:ts = stack s;	       -- take elements from stack
             s1   = s {nodes = modifyScc (nodes s) t (Right 0), stack = ts};
             acc1 = t:acc;
           } in
           if t==v                     -- make new strongly connected comp
	      then s1 {daSccs  =  acc1 : (daSccs s1)}
              else loop s1 acc1;
       } in
       loop s2 []
     else
     s2
;
                                     
scc0 []     s = reverse (daSccs s);         -- reverse to obtain correct order
scc0 (b:bs) s =                             -- visit each binding
    if visited b (nodes s) then scc0 bs s
    else scc0 bs (lowlink b s)   
;  

scc :: Eq a => [a] -> [(a,Either [a] Int)] -> [[a]];
scc bs nodes = scc0 bs (newSccState nodes); -- sort list with added dependency info into SCCs

-- e.g.
testdata = map (\ (a, b) -> (a, Left b))
          [(1, [2, 3]), (2, [3]), (3, [2]), (4, [5]), (5, [2])]
-- scc [1..5] testdata

}
